<?php

namespace Domain\Normalizer\Decorator;

use Domain\Normalizer\Normalizer;

abstract class NormalizerDecorator implements Normalizer
{
    private function __construct(
        private Normalizer $parent
    )
    {
    }

    public static function decorate(Normalizer $normalizer): static
    {
        return new static($normalizer);
    }

    abstract protected function getAdditionalHeaders(): array;

    abstract public function makePass(array $normalized): array;

    final public function getHeaders(): array
    {
        return array_merge($this->parent->getHeaders(), $this->getAdditionalHeaders());
    }

    final public function normalize(object $object): array
    {
        $normalized = $this->parent->normalize($object);
        return $this->makePass($normalized);
    }
}
